;(function ($, elementor) {
    'use strict';
    var $window = jQuery(window);

var widgetAdvancedCounter = function ($scope, $) {
    var $advancedCounter = $scope.find('.sa-advanced-counter'),
            $settings = $advancedCounter.data('settings'),
            $options = $advancedCounter.data('options');

    if (!$advancedCounter.length) {
        return;
    }

    $options.useEasing = !!$options.useEasing;
    $options.useGrouping = !!$options.useGrouping;

    elementorFrontend.waypoint($advancedCounter, function () {
        var counterInstance = new CountUp($settings.id, $settings.endValue, $options);
        if (!counterInstance.error) {
            counterInstance.start();
        } else {
            console.error(counterInstance.error);
        }
    }, {
        offset: 'bottom-in-view'
    });

};

var widgetVideoGallery = function ($scope, $) {
    var $videoGallery = $scope.find('.sa-video-gallery'),
        $playerContainer = $videoGallery.find('.sa-video-gallery-player'),
        $playlistContainer = $videoGallery.find('.sa-video-gallery-playlist'),
        $playerSettings = $videoGallery.data('player-settings'),
        $thumbsSettings = $videoGallery.data('playlist-settings');

    const Swiper = elementorFrontend.utils.swiper;
    initSwiper();
    async function initSwiper() {
        var swiperThumbs = await new Swiper($playlistContainer.find('.swiper-container'), $thumbsSettings);

        var swiperPreview = await new Swiper($playerContainer.find('.swiper-container'), {
            direction: $playerSettings.direction,
            effect: $playerSettings.effect,
            loop: $playerSettings.loop,
            speed: $playerSettings.speed,
            thumbs: {
                swiper: swiperThumbs,
            },
        });

        swiperPreview.on('slideChange', function () {
            resetVideos();
        });

    };


    var resetVideos = function () {
        $($videoGallery).find('.sa-video-player').css('z-index', -1);
        var videos = $($videoGallery).find('.sa-player-iframe');
        Array.prototype.forEach.call(videos, function (video) {
            var src = video.src;
            video.src = src.replace("?autoplay=1", "");
            $($videoGallery).find('.sa-player-iframe').prop("src", "");
        });
    };


    $('.sa-play-button').on('click', function () {
        var videoURL = $(this).data('src').split('?')[0]; // also removed @param
        var sliderWrapper = $(this).closest('.sa-player-wrapper');
        sliderWrapper.find('.sa-player-iframe').attr("src", videoURL + "?autoplay=1");
        sliderWrapper.find('.sa-video-player').css('z-index', 10);

    });
};
var widgetPricingTable = function ($scope, $) {

    var $pricingTable = $scope.find('.sa-pricing-table');

    if (!$pricingTable.length) {
        return;
    }

    var $tooltips = $pricingTable.find('.sa-tippy-tooltip'),
        widgetID = $scope.data('id');

    $tooltips.each(function (index) {
        tippy(this, {
            allowHTML: true,
            theme: 'sa-tippy-' + widgetID
        });
    });
};
var widgetFlowSlider = function ($scope, $) {
    var $flowSlider = $scope.find('.sa-flow-slider'),
        $settings = $flowSlider.data('settings'),
        $thumbsSettings = $flowSlider.data('thumbs-settings');

    if (!$flowSlider.length) {
        return;
    }
    const Swiper = elementorFrontend.utils.swiper;
    initSwiper();
    async function initSwiper() {

        var mainSlider = await new Swiper($flowSlider.find('.sa-flow-slider-main'), $settings);
        var sliderThumbs = await new Swiper($flowSlider.find('.sa-flow-slider-thumbs'), $thumbsSettings);

        mainSlider.controller.control = sliderThumbs;
        sliderThumbs.controller.control = mainSlider;

        if ($settings.pauseOnHover) {
            $($flowSlider).hover(function () {
                mainSlider.autoplay.stop();
            }, function () {
                mainSlider.autoplay.start();
            });
        }

    };
};
var widgetPaceSlider = function ($scope, $) {
    var $paceSlider = $scope.find('.sa-pace-slider'),
        $settings = $paceSlider.data('settings');

    if (!$paceSlider.length) {
        return;
    }
    const Swiper = elementorFrontend.utils.swiper;
    initSwiper();
    async function initSwiper() {

        var sliderThumbs = await new Swiper($paceSlider.find('.sa-pace-slider-thumbs'), $settings);

        if ($settings.pauseOnHover) {
            $($paceSlider).hover(function () {
                sliderThumbs.autoplay.stop();
            }, function () {
                sliderThumbs.autoplay.start();
            });
        }

    };

    $($paceSlider).find('.sa-thumb-wrapper').on('mouseenter', function () {
        var $id = '#' + $(this).data('item-index');
        $($id).siblings().removeClass('sa-active');
        $($id).addClass('sa-active');
    });

};
    var fnHanlders = {
        'sky-video-gallery.default': widgetVideoGallery,
        'sky-advanced-counter.default': widgetAdvancedCounter,
        'sky-pricing-table.default': widgetPricingTable,
        'sky-flow-slider.default': widgetFlowSlider,
        'sky-pace-slider.default': widgetPaceSlider,
    };

    $window.on('elementor/frontend/init', function () {
        $.each(fnHanlders, function (widgetName, handlerFn) {
            elementorFrontend.hooks.addAction('frontend/element_ready/' + widgetName, handlerFn);
        });
    });


}(jQuery, window.elementorFrontend));

;
(function ($) {
    'use strict';
    var $window = $(window),
        debounce = function (func, wait, immediate) {

            /**
             * 'private' variable for instance
             * The returned function will be able to reference this due to closure.
             * Each call to the returned function will share this common timer.
             */
            var timeout;

            /**
             * Calling debounce returns a new anonymous function
             */
            return function () {
                /**
                 * Reference the context and args for the setTimeout function
                 */
                var context = this,
                    args = arguments;

                /**
                 * Should the function be called now? If immediate is true and not already in a timeout then the answer is: Yes
                 * 
                 */
                var callNow = immediate && !timeout;

                /**
                 * This is the basic debounce behaviour where you can call this function several times, 
                 * But it will only execute once [before or after imposing a delay].
                 * Each time the returned function is called, the timer starts over.
                 */
                clearTimeout(timeout);

                /**
                 * Set the new timeout
                 */
                timeout = setTimeout(function () {

                    /**
                     * Inside the timeout function, clear the timeout variable which will let the next execution run when in 'immediate' mode
                     */
                    timeout = null;

                    /**
                     * Check if the function already ran with the immediate flag
                     */

                    if (!immediate) {
                        /**
                         * Call the original function with apply
                         * Apply lets you define the 'this' object as well as the arguments (both captured before setTimeout)
                         */
                        func.apply(context, args);
                    }
                }, wait);

                /**
                 * Immediate mode and no wait timer? Execute the function..
                 */
                if (callNow)
                    func.apply(context, args);
            };
        };
    $window.on('elementor/frontend/init', function () {
        var ModuleHandler = elementorModules.frontend.handlers.Base,
            DarkMode;

        DarkMode = ModuleHandler.extend({

            bindEvents: function () {
                this.run();
            },

            getDefaultSettings: function () {
                return {
                    bottom: '32px',
                    right: '32px',
                    left: 'unset',
                    time: '0.5s',
                    mixColor: '#fff',
                    backgroundColor: '#fff',
                    // buttonColorDark: '#100f2c',
                    // buttonColorLight: '#fff',
                    saveInCookies: true,
                    label: '🌓',
                    autoMatchOsTheme: false,
                };
            },

            onElementChange: debounce(function (prop) {
                if (prop.indexOf('dark_mode_') !== -1) {
                    $('.darkmode-background').remove();
                    $('.darkmode-layer').remove();
                    $('.darkmode-toggle').remove();
                    this.run();
                }
            }, 400),

            settings: function (key) {
                return this.getElementSettings('dark_mode_' + key);
            },

            run: function () {
                var options = this.getDefaultSettings(),
                    _this = this;

                if (this.settings('time.size')) {
                    options.time = (this.settings('time.size') / 1000) + 's' || '0.5s';
                }
                if (this.settings('mix_color')) {
                    options.mixColor = this.settings('mix_color') || '#fff';
                }
                if (this.settings('background_color')) {
                    options.backgroundColor = this.settings('background_color') || '#fff';
                }

                options.saveInCookies = this.settings('save_in_cookies') ? true : false;

                if (this.settings('auto_match_theme')) {
                    options.autoMatchOsTheme = true;
                }
                if (this.settings('style') && this.settings('style') == 'bulb') {
                    options.label = '💡';
                }
                if (this.settings('button_color_dark')) {
                    options.buttonColorDark = this.settings('button_color_dark') || '#100f2c';
                }
                if (this.settings('button_color_light')) {
                    options.buttonColorLight = this.settings('button_color_light') || '#fff';
                }
                if (this.settings('style') && this.settings('style') == 'dot') {
                    options.label = '';
                }
                if (this.settings('style') && (this.settings('style') == 'switcher')) {
                    $('body').addClass('sa-dark-mode-' + this.settings('style'));
                    $('body').addClass('sa-dark-mode-position-' + this.settings('toggle_position'));
                    options.label = `<span class="sa-dark-mode-slider"></span>`;
                    var leftText = this.settings('left_text') || 'Light',
                        rightText = this.settings('right_text') || 'Dark';
                    // options.right = '62px';
                }

                const darkmode = new Darkmode(options);
                darkmode.showWidget();


                /**
                 * Init by user verify
                 */
                if (window.localStorage.getItem('darkmode') === null) {
                    /**
                     * Dark mode init on load
                     */
                    if ((_this.settings('default_mode')) && (_this.settings('default_mode') == 'dark')) {
                        console.log('df');
                        $('.darkmode-toggle').trigger('click');
                    }
                    /**
                     * Auto Dark mode on this time
                     */
                    if ((_this.settings('default_mode')) && (_this.settings('default_mode') == 'auto')) {
                        var hours = new Date().getHours();
                        var isDayTime = hours > 6 && hours < 18;
                        if (isDayTime === false) {
                            $('.darkmode-toggle').trigger('click');
                        }
                    }
                }

                /**
                 * Class add after Dark Mode Init
                 */
                if (this.settings('style') && this.settings('style') == 'switcher') {
                    var leftText = this.settings('left_text') || 'Light',
                        rightText = this.settings('right_text') || 'Dark';
                    $('.darkmode-toggle').attr('data-light', leftText);
                    $('.darkmode-toggle').attr('data-night', rightText);
                }

            }
        });

        elementorFrontend.hooks.addAction('frontend/element_ready/sky-dark-mode.default', function ($scope) {
            elementorFrontend.elementsHandler.addHandler(DarkMode, {
                $element: $scope
            });
        });
    });
})(jQuery, window.elementorFrontend);
;
(function ($) {
    'use strict';
    var $window = $(window),
        debounce = function (func, wait, immediate) {

            /**
             * 'private' variable for instance
             * The returned function will be able to reference this due to closure.
             * Each call to the returned function will share this common timer.
             */
            var timeout;

            /**
             * Calling debounce returns a new anonymous function
             */
            return function () {
                /**
                 * Reference the context and args for the setTimeout function
                 */
                var context = this,
                    args = arguments;

                /**
                 * Should the function be called now? If immediate is true and not already in a timeout then the answer is: Yes
                 * 
                 */
                var callNow = immediate && !timeout;

                /**
                 * This is the basic debounce behaviour where you can call this function several times, 
                 * But it will only execute once [before or after imposing a delay].
                 * Each time the returned function is called, the timer starts over.
                 */
                clearTimeout(timeout);

                /**
                 * Set the new timeout
                 */
                timeout = setTimeout(function () {

                    /**
                     * Inside the timeout function, clear the timeout variable which will let the next execution run when in 'immediate' mode
                     */
                    timeout = null;

                    /**
                     * Check if the function already ran with the immediate flag
                     */

                    if (!immediate) {
                        /**
                         * Call the original function with apply
                         * Apply lets you define the 'this' object as well as the arguments (both captured before setTimeout)
                         */
                        func.apply(context, args);
                    }
                }, wait);

                /**
                 * Immediate mode and no wait timer? Execute the function..
                 */
                if (callNow)
                    func.apply(context, args);
            };
        };
    $window.on('elementor/frontend/init', function () {
        var ModuleHandler = elementorModules.frontend.handlers.Base,
            AdvancedTooltip;

        AdvancedTooltip = ModuleHandler.extend({

            bindEvents: function () {
                this.run();
            },

            getDefaultSettings: function () {
                return {
                    allowHTML: true,
                    animateFill: false,
                    delay: 0,
                };
            },

            settings: function (key) {
                return this.getElementSettings('sa_ad_tooltip_' + key);
            },

            onElementChange: debounce(function (prop) {
                if (prop.indexOf('sa_ad_tooltip_') !== -1) {
                    this.instance.destroy();
                    this.run();
                }
            }, 400),

            run: function () {
                var options = this.getDefaultSettings(),
                    elementID = this.getID(),
                    elementContainer = document.querySelector('.elementor-element-' + elementID + ' .elementor-widget-container');

                if (this.settings('content')) {
                    options.content = this.settings('content');
                }

                options.arrow = !!this.settings('arrow');
                options.followCursor = !!this.settings('follow_cursor');

                if (this.settings('placement')) {
                    options.placement = this.settings('placement');
                }

                if (this.settings('trigger')) {
                    options.trigger = this.settings('trigger');
                }

                if (this.settings('trigger_custom') == 'yes' && this.settings('trigger_custom_selector')) {
                    options.triggerTarget = document.querySelector(this.settings('trigger_custom_selector'));
                }

                if (this.settings('animation')) {
                    if (this.settings('animation') == 'fill') {
                        options.animateFill = true;
                    } else {
                        options.animation = this.settings('animation');
                    }
                }

                if (this.settings('offset_toggle')) {
                    if (this.settings('offset_x.size') || this.settings('offset_y.size')) {
                        options.offset = [this.settings('offset_x.size') || 0, this.settings('offset_y.size') || 0];
                    }
                }
                if (this.settings('delay_toggle')) {
                    if (this.settings('delay_show.size') || this.settings('delay_hide.size')) {
                        options.delay = [this.settings('delay_show.size') || null, this.settings('delay_hide.size') || null];
                    }
                }
                if (this.settings('anim_duration_toggle')) {
                    if (this.settings('anim_duration_show.size') || this.settings('anim_duration_hide.size')) {
                        options.duration = [this.settings('anim_duration_show.size') || null, this.settings('anim_duration_hide.size') || null];
                    }
                }

                /**
                 * Start hide_on_click
                 */
                if (this.settings('hide_on_click') === 'false') {
                    options.hideOnClick = false;
                }
                if (this.settings('hide_on_click') === 'toggle') {
                    options.hideOnClick = 'toggle';
                }
                /**
                 * End hide_on_click
                 */

                if (this.isEdit !== true) {
                    options.interactive = !!this.settings('interactive');
                }

                if (this.settings('enable')) {
                    options.theme = 'sa-ad-tippy-' + elementID;
                    this.instance = tippy(elementContainer, options);

                }

            }
        });


        elementorFrontend.hooks.addAction('frontend/element_ready/widget', function ($scope) {
            elementorFrontend.elementsHandler.addHandler(AdvancedTooltip, {
                $element: $scope
            });
        });
    });

}(jQuery));
;
(function ($) {
    'use strict';
    var $window = $(window),
        debounce = function (func, wait, immediate) {

            /**
             * 'private' variable for instance
             * The returned function will be able to reference this due to closure.
             * Each call to the returned function will share this common timer.
             */
            var timeout;

            /**
             * Calling debounce returns a new anonymous function
             */
            return function () {
                /**
                 * Reference the context and args for the setTimeout function
                 */
                var context = this,
                    args = arguments;

                /**
                 * Should the function be called now? If immediate is true and not already in a timeout then the answer is: Yes
                 * 
                 */
                var callNow = immediate && !timeout;

                /**
                 * This is the basic debounce behaviour where you can call this function several times, 
                 * But it will only execute once [before or after imposing a delay].
                 * Each time the returned function is called, the timer starts over.
                 */
                clearTimeout(timeout);

                /**
                 * Set the new timeout
                 */
                timeout = setTimeout(function () {

                    /**
                     * Inside the timeout function, clear the timeout variable which will let the next execution run when in 'immediate' mode
                     */
                    timeout = null;

                    /**
                     * Check if the function already ran with the immediate flag
                     */

                    if (!immediate) {
                        /**
                         * Call the original function with apply
                         * Apply lets you define the 'this' object as well as the arguments (both captured before setTimeout)
                         */
                        func.apply(context, args);
                    }
                }, wait);

                /**
                 * Immediate mode and no wait timer? Execute the function..
                 */
                if (callNow)
                    func.apply(context, args);
            };
        };
    $window.on('elementor/frontend/init', function () {
        var ModuleHandler = elementorModules.frontend.handlers.Base,
            ParticleEffects;

        ParticleEffects = ModuleHandler.extend({

            bindEvents: function () {
                this.run();
            },

            settings: function (key) {
                return this.getElementSettings('sa_particles_' + key);
            },

            onElementChange: debounce(function (prop) {
                if (prop.indexOf('sa_particles_') !== -1) {
                    this.run();
                }
            }, 400),

            run: function () {
                var $mainElement = this.$element,
                    elementID = this.getID(),
                    elementContainer = $('.elementor-element-' + elementID),
                    particlesID = 'sa-particles-el-' + elementID,
                    particles_style = this.settings('style');

                if (!$mainElement.hasClass("sa-particles-yes")) {
                    return;
                }
                if (!this.settings('enable')) {
                    return;
                }

                if ($('#' + particlesID).length === 0) {
                    $(elementContainer).prepend('<div id="' + particlesID + '" class="sa-particles-el-wrapper"></div>');
                }

                if (particles_style == 'custom' && this.settings('custom_json') !== "") {
                    var custom_json = JSON.parse(this.settings('custom_json'));
                    particlesJS(particlesID, custom_json);
                } else if (particles_style !== 'custom') {
                    var
                        number_value = 80,
                        shape_type = 'circle',
                        shape_nb_sides = 5,
                        opacity_value = 0.5,
                        opacity_random = true,
                        opacity_anim_enable = false,
                        size_value = 3,
                        size_random = true,
                        size_anim_enable = false,
                        line_linked_enable = false,
                        move_speed = 6,
                        move_direction = "none",
                        move_random = true,
                        onhover = "repulse",
                        interactive = false;

                    if ('polygon' == particles_style) {
                        line_linked_enable = true;
                        opacity_random = false;
                        move_random = false;
                        move_speed = 6;
                    } else if ('nasa' == particles_style) {
                        number_value = 160;
                        shape_type = "circle";
                        opacity_value = 0.7;
                        opacity_anim_enable = true;
                        move_speed = 1;
                        size_value = 3;
                        onhover = "bubble";
                    } else if ('bubble' == particles_style) {
                        move_speed = 8;
                        move_direction = this.settings('move_direction') || "none";
                        number_value = 6;
                        size_value = 160;
                        shape_type = "polygon";
                        size_random = false;
                        opacity_value = 0.3;
                        opacity_random = true;
                    } else if ('snow' == particles_style) {
                        opacity_value = 0.8;
                        opacity_random = false;
                        size_value = 6;
                        move_speed = 3;
                        move_direction = this.settings('move_direction');
                        number_value = 200;
                    }


                    var particles_color = this.settings('color') || '#d9d9d9';

                    if (this.settings('opacity.size') != '' || this.settings('opacity.size') == '0') {
                        opacity_value = this.settings('opacity.size');
                    }

                    if (this.settings('number') && this.settings('number.size') !== '') {
                        number_value = this.settings('number.size');
                    }

                    if (this.settings('size') && this.settings('size.size') !== '') {
                        size_value = this.settings('size.size');
                    }

                    if (this.settings('move_speed') && this.settings('move_speed.size') !== '') {
                        move_speed = this.settings('move_speed.size');
                    }

                    if (this.settings('interactive') && this.settings('interactive') == 'yes') {
                        interactive = true;
                    }


                    if (this.settings('z_index')) {
                        $('#' + particlesID).css('z-index', this.settings('z_index'));
                    }


                    var configParticles = {
                        "particles": {
                            "number": {
                                "value": number_value,
                                "density": {
                                    "enable": true,
                                    "value_area": 800
                                }
                            },
                            "color": {
                                "value": particles_color
                            },
                            "shape": {
                                "type": shape_type,
                                "stroke": {
                                    "width": 0,
                                    "color": "#000000"
                                },
                                "polygon": {
                                    "nb_sides": shape_nb_sides
                                },
                            },
                            "opacity": {
                                "value": opacity_value,
                                "random": opacity_random,
                                "anim": {
                                    "enable": opacity_anim_enable,
                                    "speed": 1,
                                    "opacity_min": 0.1,
                                    "sync": false
                                }
                            },
                            "size": {
                                "value": size_value,
                                "random": size_random,
                                "anim": {
                                    "enable": size_anim_enable,
                                    "speed": 40,
                                    "size_min": 0.1,
                                    "sync": false
                                }
                            },
                            "line_linked": {
                                "enable": line_linked_enable,
                                "distance": 150,
                                "color": particles_color,
                                "opacity": 0.4,
                                "width": 1
                            },
                            "move": {
                                "enable": true,
                                "speed": move_speed,
                                "direction": move_direction,
                                "random": move_random,
                                "straight": false,
                                "out_mode": "out",
                                "bounce": false,
                                "attract": {
                                    "enable": false,
                                    "rotateX": 600,
                                    "rotateY": 1200
                                }
                            }
                        },
                        "interactivity": {
                            "detect_on": "canvas",
                            "events": {
                                "onhover": {
                                    "enable": interactive,
                                    "mode": onhover
                                },
                                "onclick": {
                                    "enable": true,
                                    "mode": "push"
                                },
                                "resize": true
                            },
                            "modes": {
                                "grab": {
                                    "distance": 400,
                                    "line_linked": {
                                        "opacity": 1
                                    }
                                },
                                "bubble": {
                                    "distance": 400,
                                    "size": 40,
                                    "duration": 2,
                                    "opacity": 8,
                                    "speed": 3
                                },
                                "repulse": {
                                    "distance": 200,
                                    "duration": 0.4
                                },
                                "push": {
                                    "particles_nb": 4
                                },
                                "remove": {
                                    "particles_nb": 2
                                }
                            }
                        },
                        "retina_detect": true
                    };

                    particlesJS(particlesID, configParticles);

                }

            }
        });


        elementorFrontend.hooks.addAction('frontend/element_ready/section', function ($scope) {
            elementorFrontend.elementsHandler.addHandler(ParticleEffects, {
                $element: $scope
            });
        });
    });

}(jQuery));
;
(function ($) {
    'use strict';
    var $window = $(window),
        debounce = function (func, wait, immediate) {

            /**
             * 'private' variable for instance
             * The returned function will be able to reference this due to closure.
             * Each call to the returned function will share this common timer.
             */
            var timeout;

            /**
             * Calling debounce returns a new anonymous function
             */
            return function () {
                /**
                 * Reference the context and args for the setTimeout function
                 */
                var context = this,
                    args = arguments;

                /**
                 * Should the function be called now? If immediate is true and not already in a timeout then the answer is: Yes
                 * 
                 */
                var callNow = immediate && !timeout;

                /**
                 * This is the basic debounce behaviour where you can call this function several times, 
                 * But it will only execute once [before or after imposing a delay].
                 * Each time the returned function is called, the timer starts over.
                 */
                clearTimeout(timeout);

                /**
                 * Set the new timeout
                 */
                timeout = setTimeout(function () {

                    /**
                     * Inside the timeout function, clear the timeout variable which will let the next execution run when in 'immediate' mode
                     */
                    timeout = null;

                    /**
                     * Check if the function already ran with the immediate flag
                     */

                    if (!immediate) {
                        /**
                         * Call the original function with apply
                         * Apply lets you define the 'this' object as well as the arguments (both captured before setTimeout)
                         */
                        func.apply(context, args);
                    }
                }, wait);

                /**
                 * Immediate mode and no wait timer? Execute the function..
                 */
                if (callNow)
                    func.apply(context, args);
            };
        };
    $window.on('elementor/frontend/init', function () {
        var ModuleHandler = elementorModules.frontend.handlers.Base,
            ConfettiEffects;

        ConfettiEffects = ModuleHandler.extend({

            bindEvents: function () {
                this.run();
            },

            getDefaultSettings: function () {
                return {
                    resize: true,
                    useWorker: true,
                };
            },

            settings: function (key) {
                return this.getElementSettings('sa_confetti_ef_' + key);
            },
            onElementChange: debounce(function (prop) {
                if (prop.indexOf('sa_confetti_ef_') !== -1) {
                    // this.instance.destroy();
                    this.run();
                }
            }, 400),

            run: function () {
                var options = this.getDefaultSettings(),
                    $element = this.$element;

                if (this.settings('enable') !== 'yes') {
                    return;
                }

                if (this.settings('particle_count.size')) {
                    options.particleCount = this.settings('particle_count.size') || 100;
                }

                if (this.settings('angle.size')) {
                    options.angle = this.settings('angle.size') || 90;
                }

                if (this.settings('spread.size')) {
                    options.spread = this.settings('spread.size') || 45;
                }

                if (this.settings('start_velocity.size')) {
                    options.startVelocity = this.settings('start_velocity.size') || 45;
                }

                if (this.settings('decay.size')) {
                    options.decay = this.settings('decay.size') || 0.9;
                }

                if (this.settings('gravity.size')) {
                    options.gravity = this.settings('gravity.size') || 1;
                }

                if (this.settings('drift.size')) {
                    options.drift = this.settings('drift.size') || 0;
                }

                if (this.settings('origin')) {
                    if (this.settings('origin_x.size') || this.settings('origin_y.size')) {
                        options.origin = {
                            x: this.settings('origin_x.size') || 0.5,
                            y: this.settings('origin_y.size') || 0.6
                        }
                    }
                }

                if (this.settings('colors')) {
                    var colors = this.settings('colors');
                    options.colors = colors.split(',');
                }
                if (this.settings('shapes')) {
                    var shapes = this.settings('shapes');
                    options.shapes = shapes.split(',');
                }

                if (this.settings('scalar.size')) {
                    options.scalar = this.settings('scalar.size') || 1;
                }

                var _instance = this;

                var Confetti = {
                    randomInRange: function (min, max) {
                        return Math.random() * (max - min) + min;
                    },
                    effectsBasic: function () {
                        /**
                         * Execute Confetti with Options
                         */
                        confetti(options);
                    },
                    effectsRandom: function () {
                        options.angle = Confetti.randomInRange(55, _instance.settings('angle.size') || 90);
                        options.spread = Confetti.randomInRange(50, _instance.settings('spread.size') || 70);
                        options.particleCount = Confetti.randomInRange(55, _instance.settings('particle_count.size') || 100);
                        /**
                         * Execute Confetti with Options
                         */
                        confetti(options);
                    },
                    effectsFireWorks: function () {
                        var duration = _instance.settings('anim_dur.size') || 5000;
                        var animationEnd = Date.now() + duration;
                        var defaults = {
                            startVelocity: _instance.settings('start_velocity.size') || 30,
                            spread: _instance.settings('spread.size') || 360,
                            shapes: _instance.settings('shapes.size') ? shapes.split(',') : ['circle', 'circle', 'square'],
                            ticks: 60,
                            zIndex: 0
                        };

                        if (_instance.settings('angle.size')) {
                            defaults.angle = _instance.settings('angle.size') || 90;
                        }
                        if (_instance.settings('decay.size')) {
                            defaults.decay = _instance.settings('decay.size') || 0.9;
                        }
                        if (_instance.settings('gravity.size')) {
                            defaults.gravity = _instance.settings('gravity.size') || 1;
                        }
                        if (_instance.settings('drift.size')) {
                            defaults.drift = _instance.settings('drift.size') || 0;
                        }
                        if (_instance.settings('scalar.size')) {
                            defaults.scalar = _instance.settings('scalar.size') || 1;
                        }

                        let particles = 50;
                        if (_instance.settings('particle_count.size')) {
                            particles = _instance.settings('particle_count.size') || 50;
                        }

                        var interval = setInterval(function () {
                            var timeLeft = animationEnd - Date.now();

                            if (timeLeft <= 0) {
                                return clearInterval(interval);
                            }

                            var particleCount = particles * (timeLeft / duration);
                            // since particles fall down, start a bit higher than random
                            /**
                             * Execute Confetti with Options
                             */
                            confetti(Object.assign({}, defaults, {
                                particleCount,
                                origin: {
                                    x: Confetti.randomInRange(0.1, 0.3),
                                    y: Math.random() - 0.2
                                }
                            }));
                            confetti(Object.assign({}, defaults, {
                                particleCount,
                                origin: {
                                    x: Confetti.randomInRange(0.7, 0.9),
                                    y: Math.random() - 0.2
                                }
                            }));
                        }, 250);
                    },
                    effectsSchoolPride: function () {
                        var duration = _instance.settings('anim_dur.size') || 5000;
                        var end = Date.now() + (duration);

                        (function frame() {
                            /**
                             * Execute Confetti with Options
                             */
                            var defaults = {
                                particleCount: _instance.settings('particle_count.size') || 2,
                                angle: _instance.settings('angle.size') || 60,
                                spread: _instance.settings('spread.size') || 55,
                                shapes: _instance.settings('shapes.size') ? shapes.split(',') : ['circle', 'circle', 'square'],
                                origin: {
                                    x: 0
                                },
                                colors: colors.split(',')
                            };

                            var defaults2 = {
                                particleCount: _instance.settings('particle_count.size') || 2,
                                angle: (_instance.settings('angle.size') || 60) * 2, //120
                                spread: _instance.settings('spread.size') || 55,
                                shapes: _instance.settings('shapes.size') ? shapes.split(',') : ['circle', 'circle', 'square'],
                                origin: {
                                    x: 1
                                },
                                colors: colors.split(',')
                            };

                            if (_instance.settings('decay.size')) {
                                defaults.decay = _instance.settings('decay.size') || 0.9;
                                defaults2.decay = _instance.settings('decay.size') || 0.9;
                            }

                            if (_instance.settings('gravity.size')) {
                                defaults.gravity = _instance.settings('gravity.size') || 1;
                                defaults2.gravity = _instance.settings('gravity.size') || 1;
                            }

                            if (_instance.settings('drift.size')) {
                                defaults.drift = _instance.settings('drift.size') || 0;
                                defaults2.drift = _instance.settings('drift.size') || 0;
                            }

                            if (_instance.settings('scalar.size')) {
                                defaults.scalar = _instance.settings('scalar.size') || 1;
                                defaults2.scalar = _instance.settings('scalar.size') || 1;
                            }

                            confetti(defaults);
                            confetti(defaults2);

                            if (Date.now() < end) {
                                requestAnimationFrame(frame);
                            }
                        }());
                    },
                    effectsSnow: function () {
                        var duration = _instance.settings('anim_dur.size') || 10000;
                        var animationEnd = Date.now() + duration;
                        var skew = 1;

                        (function frame() {
                            var timeLeft = animationEnd - Date.now();
                            var ticks = Math.max(200, 500 * (timeLeft / duration));
                            skew = Math.max(0.8, skew - 0.001);

                            var defaults = {
                                particleCount: _instance.settings('particle_count.size') || 1,
                                startVelocity: _instance.settings('start_velocity.size') || 0,
                                ticks: ticks,
                                origin: {
                                    x: Math.random(),
                                    // since particles fall down, skew start toward the top
                                    y: (Math.random() * skew) - 0.2
                                },
                                colors: colors.split(','),
                                shapes: _instance.settings('shapes.size') ? shapes.split(',') : ['circle'],
                                gravity: Confetti.randomInRange(0.4, 0.6),
                                scalar: Confetti.randomInRange(0.4, _instance.settings('scalar.size') || 1),
                                drift: Confetti.randomInRange(-0.4, 0.4)
                            };

                            if (_instance.settings('angle.size')) {
                                defaults.angle = _instance.settings('angle.size') || 90;
                            }
                            if (_instance.settings('spread.size')) {
                                defaults.spread = _instance.settings('spread.size') || 45;
                            }
                            if (_instance.settings('decay.size')) {
                                defaults.decay = _instance.settings('decay.size') || 0.9;
                            }

                            /**
                             * Execute Confetti with Options
                             */
                            confetti(defaults);

                            if (timeLeft > 0) {
                                requestAnimationFrame(frame);
                            }
                        }());
                    },
                    effects: function () {
                        if (_instance.settings('type') == 'basic') {
                            this.effectsBasic();
                        }
                        if (_instance.settings('type') == 'random') {
                            this.effectsRandom();
                        }
                        if (_instance.settings('type') == 'fireworks') {
                            this.effectsFireWorks();
                        }
                        if (_instance.settings('type') == 'school_pride') {
                            this.effectsSchoolPride();
                        }
                        if (_instance.settings('type') == 'snow') {
                            this.effectsSnow();
                        }
                    },
                    initEffects: function () {
                        var delay = 10;
                        if (_instance.settings('delay.size')) {
                            delay = _instance.settings('delay.size') || 10;
                        }
                        setTimeout(() => {
                            Confetti.effects();
                        }, delay);
                    },
                    smartTrigger: function () {
                        var $triggerType = _instance.settings('trigger_type');
                        if ($triggerType == 'load') {
                            Confetti.initEffects();
                        }
                        if ($triggerType == 'on_view') {
                            elementorFrontend.waypoint($element, function () {
                                Confetti.initEffects();
                            }, {
                                // offset: 'bottom-in-view',
                                offset: '80%'
                            });
                        }
                        if ($triggerType == 'click' || $triggerType == 'mouseenter') {
                            let triggerSelector = _instance.settings('trigger_selector');
                            if ($(_instance.settings('trigger_selector')).length <= 0) {
                                // return;
                                /**
                                 * Override blank by Own Object
                                 * No need return
                                 * Set default own Element
                                 */
                                triggerSelector = $element;
                            }
                            
                            $(triggerSelector).on($triggerType, function () {
                                Confetti.initEffects();
                            });
                        }
                        if ($triggerType == 'ajax_success') {
                            $(document).ajaxComplete(function (event, jqxhr, settings) {
                                Confetti.initEffects();
                            });
                        }

                    },
                    init: function () {
                        $(document).ready(function () {
                            Confetti.smartTrigger();
                        });
                    }
                }

                // kick-off confetti
                Confetti.init();

            }
        });


        elementorFrontend.hooks.addAction('frontend/element_ready/widget', function ($scope) {
            elementorFrontend.elementsHandler.addHandler(ConfettiEffects, {
                $element: $scope
            });
        });

        elementorFrontend.hooks.addAction('frontend/element_ready/section', function ($scope) {
            elementorFrontend.elementsHandler.addHandler(ConfettiEffects, {
                $element: $scope
            });
        });

    });

}(jQuery));